<?php

namespace App\Http\Controllers\Member;

use App\Http\Controllers\Controller;
use App\Models\About;
use App\Models\Activity;
use App\Models\ActivityApply;
use App\Models\Agent;
use App\Models\Api;
use App\Models\ApiGame;
use App\Models\AsideAdv;
use App\Models\Banner;
use App\Models\QuickUrl;
use App\Models\GameList;
use App\Models\Member;
use App\Models\SystemConfig;
use App\Models\SystemNotice;
use App\Services\ActivityService;
use App\Services\JzService;
use App\Services\SelfService;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\Rule;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;

// 游客访问
class IndexController extends MemberBaseController
{
    // 获取 banner 数据
    public function getBannerListByGroups(Request $request)
    {

        $data = Banner::query()
            ->select(['title', 'url', 'groups', 'dimensions', 'weight'])
            //->where('is_open',1)->where('groups',$request->get('group', ''))

            ->where($this->convertWhere([
                'is_open' => true,
                'groups' => $request->get('group', '')
            ]))

            ->orderByDesc('weight')
            ->get();

        return $this->success(['data' => $data]);
    }

    // 获取首页游戏分类数据（手机）
    public function getMainGameList(){
        $json = SystemConfig::query()->getConfigValue('mobile_category_json');
        // 只返回开放的数据
        $category = collect(json_decode($json, 1))
            ->where('is_open','true')
            ->sortByDesc('weight');
            //->only(['title','icon']);
        
        $collection = ApiGame::query()->isMobile(1)->displayFormatter();

            // $this->api_print($collection->toArray());

        $online_apis = Api::where('is_open',1)->pluck('api_name');

        $collection = $collection
            ->whereIn('api_name',$online_apis)
            ->whereIn('game_type_text', $category->pluck('title'))
            ->groupBy(function($item,$key){
                return $item['game_type_text'];
            });

        $result = [];

        foreach($category as $value){
            $temp = [
                'title' => $value['title'],
                'icon' => $value['icon'],
            ];
            if($value['title'] == '热门'){
                $temp['child'] = ApiGame::query()->isMobile(1)->whereTags(['hot'])->whereIn('api_name',$online_apis)->displayFormatter()->toArray();
            }else{
                $temp['child'] = $collection->get($value['title']);
            }
            array_push($result, $temp);
        }

        return $this->success(['data' => $result]);
    }

    public function getAllApiGames(){
        $data = ApiGame::select(['title','api_name','class_name','game_type','params','weight','is_open','tags'])
            ->where('is_open',1)
            ->whereIn('api_name',Api::where('is_open',1)->pluck('api_name'))
            ->orderBy('weight','desc')->get();

        return $this->success(['data' => $data]);
    }

    // 获取 system_config 数据
    public function getSystemConfig(Request $request){
        $group = $request->get('group','system');
        // 默认是 system group
        $data = SystemConfig::query()->getConfigGroup($group);

        $url = $request->get('url');
        if($group == 'system' && $url){
            $agent = Agent::where(function($query) use($url){
                $query->where('agent_pc_uri','like','%'.$url.'%')->orWhere('agent_wap_uri','like','%'.$url.'%');
            })->first();

            if($agent){
                $data['site_mobile'] = $agent->agent_wap_uri ?? $data['site_mobile'];
                $data['site_pc'] = $agent->agent_pc_uri ?? $data['site_pc'];
            }

        }

        return $this->success(['data' => $data]);
    }

    // 获取 system_notice数据
    public function getSystemNotice(Request $request)
    {
        $data = SystemNotice::query()
            ->select(['title', 'content', 'url'])
            ->groupName('首页')
            ->where($this->convertWhere([
                'is_open' => true
            ]))
            ->orderByDesc('weight')
            ->get();
        return $this->success(['data' => $data]);
    }

    public function getAppNotice(){
        $data = SystemNotice::query()->select(['title','text_content','url'])
            ->where($this->convertWhere([
                'is_open' => true,
                'is_app' => true
            ]))
            ->orderByDesc('weight')->get();
        return $this->success(['data' => $data]);
    }

    // 获取首页的热门游戏

    // 获取活动类型
    public function getActivityType(){
        return $this->success(['data' => config('platform.activity_type')]);
    }

    // 获取活动数据 ,包括封面图片，id
    public function getActivityList()
    {
        $data = Activity::query()
            ->where('is_open', 1)
            ->isApp()
            ->where('cover_image','!=','')
            ->orderByDesc('weight')
            ->latest()
            ->get(['id','title','subtitle','cover_image','type','weight']);
        return $this->success(['data' => $data]);
    }

    // 获取 activity 详情
    public function getActivityDetail(Activity $activity,Request $request){
        //return $this->success(['data' => $activity->append('type_text')]);
        return app(ActivityService::class)->getActivityDetailHtml($activity);
    }

    // 获取所有电子游戏类型和封面图

    // 根据 游戏类型 获取电子游戏列表

    // 获取 about 数据
    public function getAbouts(Request $request)
    {
        $data = About::query()
            ->where('is_open', 1)
            ->where('type',$request->get('type',1))
            ->orderByDesc('weight')->firstOrFail();
        return $this->success(['data' => $data]);
    }

    // 获取除了电子之外的游戏，如果Api表中关闭，那么都不显示
    public function getGameApiList(Request $request){
        if(!$gameType = $request->get('gameType')) return $this->failed('gameType参数不能为空');

        $isMobile = $request->get('isMobile',0);

        $data = DB::table('api_games')->select('api_games.*')->join('apis',function($join) {
            $join->on('apis.api_name','=','api_games.api_name')->where('apis.is_open',1);
        })->where('api_games.game_type',$gameType)
            ->when($isMobile,function($query) use ($isMobile){
                $query->whereIn('client_type',$isMobile ? [0,2] : [0,1]);
            })
            ->where('api_games.is_open',1)
            ->orderBy('api_games.weight','desc')
            ->latest()
            ->get();

        return $this->success(['data' => $data]);
    }

    // 获取 game_lists 中的游戏
    public function getGameLists(Request $request){
        if(!$gameType = $request->get('gameType')) return $this->failed('gameType参数不能为空');

        $tag = $request->get('tag','');
        $tag = ($tag && $tag != 'all')? [$tag] : [];

        if($gameType == 2){
            $mod = ApiGame::whereIn('api_name',Api::where('is_open',1)->pluck('api_name'))
                ->isMobile($request->get('isMobile',0))
                ->when($request->get('api_code'),function($query) use($request){
                    $query->where('api_name',$request->get('api_code'));
                })
                ->when($request->get('keyword',''),function($query) use($request){
                    $query->where('title','like','%'.$request->get('keyword').'%');
                })->where('game_type',$gameType)
                ->whereTags($tag)->where('is_open',1)->orderBy('weight','desc');

            $data = $request->get('isMobile') ? $mod->get() : $mod->paginate($request->get('limit',15));
            return $this->success(['data' => $data]);
        }

        $mod = GameList::whereIn('api_name', Api::where('is_open',1)->pluck('api_name'))
            ->with('api:api_name,api_title')->when($request->get('api_code'),function($query) use($request){
                $query->where('api_name',$request->get('api_code'));
            })
            ->when($request->get('isMobile'),function($query) use($request) {
                $query->whichClientType($request->get('isMobile') ? 2 : 1);
            })
            ->when($request->get('keyword',''),function($query) use($request){
                $query->where('name','like','%'.$request->get('keyword').'%');
            })
            ->where('game_type',$gameType)
            ->whereTags($tag)
            ->where('is_open',1)->orderBy('weight','desc');
            //->paginate($request->get('limit',15));

        $data = $request->get('isMobile') ? $mod->get() : $mod->paginate($request->get('limit',15));

        return $this->success(['data' => $data]);
    }

    public function getSlotLogoList(Request $request){
        if(!$gameType = $request->get('gameType')) return $this->failed('gameType参数不能为空');

        $data = ApiGame::where('game_type',$gameType)->where('is_open',1)->pluck($request->get('isMobile') ? 'mobile_pic':'web_pic','api_name');
        return $this->success(['data' => $data]);
    }

    // 手机端

    // 获取首页所有游戏列表（包括游戏地址，游戏图标，游戏名称），按游戏种类分类，
    public function game_type(Request $request){
        // 获取系统彩的游戏名称
        $sys_cp = Api::where('api_name','LY')->first();

        $data = collect(config('platform.game_type'))->map(function($item, $key) use ($sys_cp){
            $data = [];
            $data['key'] = $key;
            $data['value'] = ($key == 99 && $sys_cp && $sys_cp->api_title) ? $sys_cp->api_title : $item;
            $data['isLobbyPage'] = $key == 3 || $key == 6;
            return $data;
        });
        return $this->success(['data' => array_values($data->toArray())]);
    }

    public function getAsideList(){
        $adv_ids = AsideAdv::query()->where('is_open',1)->pluck('id','group')->toArray();

        $advs = AsideAdv::query()
            ->select(['vertical','horizontal','group','url_id','effect'])
            ->with('advs:pic_url,pic_height,pic_width,group')
            ->whereIn('id',array_values($adv_ids))->get();

        $advs->transform(function($item){
            $item->full_url = $item->url_id ? $item->quickurl->full_url : '';
            return $item;
        });

        return $this->success(['data' => $advs],'');
    }

    public function getCommonLink(){
        $data = QuickUrl::opened()->get()->makeHidden(['created_at','updated_at','is_open']);
        return $this->success(['data' => $data]);
    }

    // 首页热门游戏的列表
    public function getMainPageHotSlotGame(){
        $apis = ['FMG','BBIN','IMPT','AG','JDB','HB','ISB','CQ9','KY'];

        $apilists = Api::whereIn('api_name',$apis)->get(['api_title','api_name']);

        $sql = '(SELECT a.* FROM game_lists AS a WHERE (SELECT COUNT(*) FROM game_lists AS b WHERE b.api_name = a.api_name AND b.id >= a.id ) < 11 AND a.tags LIKE "%new%" ORDER BY a.api_name ASC,a.weight desc) cc';
        // $data = \DB::select('SELECT a.* FROM game_lists AS a WHERE (SELECT COUNT(*) FROM game_lists AS b WHERE b.api_name = a.api_name AND b.id >= a.id ) < 11 AND a.tags LIKE "%new%" ORDER BY a.api_name ASC,a.weight desc');
        $data = \DB::table(\DB::raw($sql))->get();
        /*
        ->transform(function($item){
            $item->img_path = getUrlByDomain($item->img_path);
            return $item;
        }); */
        // $data = collect($data);

        // $data = $data->whereIn('api_name',$apis)->groupBy('api_name');
        $return = $apilists->map(function($item) use ($data){
            $item->list = $data->where('api_name',$item->api_name);
            return $item;
        });

        return $this->success(['data' => $return]);
    }

    // app(App\Http\Controllers\Member\IndexController::class)->lotterylist()
    public function lotterylist(){
        return $this->lotteryMethod('gamelist');
    }

    public function lotteryhot(){
        return $this->lotteryMethod('hotlist',\request()->only('count'));
    }

    public function lotteryMethod($method,$params = []){
        $json = app(JzService::class)->$method($params);
        if(!isJson($json)) return $this->failed('获取数据错误');

        $data = json_decode($json,1);
        if(Arr::get($data,'code') != '200') return $this->failed($data['msg']);

        return $this->success(['data' => $data['data'],'url' => app(JzService::class)->getApiDomain()]);
    }
}
